import java.util.ArrayList;
import java.util.Scanner;
import java.io.File;
import java.io.FileNotFoundException;

class DataFrame {
    /**
     * 
     */
    public DataFrame() {
	names = new ArrayList<String>();
	examples = new ArrayList<Example>();
	bounds = new ArrayList<MinMax>();
    }
	
    /**
     * 
     */
    public DataFrame(String fileName) {
	names = new ArrayList<String>();
	examples = new ArrayList<Example>();
	bounds = new ArrayList<MinMax>();
	readFile(fileName);
    }
	
    /**
     * 
     */
    public void readFile(String fileName) {
	try {
	    File f = new File(fileName);
	    Scanner scanner = new Scanner(f);
	    
	    String name;
			
	    // first line : names of attributes
	    Scanner sc = new Scanner(scanner.nextLine());
	    while(sc.hasNext()) {
		names.add(sc.next());
	    }
	    sc.close();

	    // examples
	    while(scanner.hasNext()) {
		sc = new Scanner(scanner.nextLine());
		Example example = new Example();
		while(sc.hasNext()) {
		    if (sc.hasNextDouble()) {
			example.add(sc.nextDouble());
		    } else {
			System.out.println("Not Found :" + sc.next());			
		    }
		}
		sc.close();
		examples.add(example);
	    }
	    
	    scanner.close();

	    for(int i = 0; i < names.size(); i++) {
		bounds.add(new MinMax(min(i), max(i)));
	    }

	} catch (FileNotFoundException exception) {
	    System.out.println("File not found");
	}
    }
	
	
    /**
     * 
     */
    public void add(Example ex) {
	examples.add(ex);
    }
	
    /**
     * Number of examples
     */
    public int size() {
	return examples.size();
    }
	
    /**
     * 
     */
    public void setNames(ArrayList<String> n) {
	names = n;
    }
	
    /**
     * 
     */
    public ArrayList<String> getNames() {
	return names;
    }
	
    /**
     * 
     */
    public Example getExample(int i) {
	return examples.get(i);
    }

    /**
     * 
     */
    public ArrayList<Example> getExamples() {
	return examples;
    }

    /**
     * 
     */
    public DataFrame select(int attribute, double value) {
	DataFrame df = new DataFrame();
	df.setNames(names);
		
	for(Example example : examples)
	    if (example.get(attribute) == value)
		df.add(example);
		
	return df;
    }
	
	
    /**
     *  Min of column
     */
    public double min(int attribute) {
	double m = examples.get(0).get(attribute);

	for(Example example : examples)
	    if (example.get(attribute) < m)
		m = example.get(attribute);
		
	return m;
    }
	
    /**
     *  Max of column
     */
    public double max(int attribute) {
	double m = examples.get(0).get(attribute);

	for(Example example : examples)
	    if (m < example.get(attribute))
		m = example.get(attribute);
		
	return m;
    }
		
    /**
     * 
     */
    public String toString() {
	String res = "";
		
	if (names.size() > 0) {
	    for(String name : names)
		res += name + " ";
	    res += "\n";
			
	    for(Example example : examples)
		res += example.toString() + "\n";
	}
		
	return res;
    }
	
    // Names of the attributes
    protected ArrayList<String> names;
	
    // List of the Examples
    protected ArrayList<Example> examples;

    // bounds for each attributes
    public ArrayList<MinMax> bounds;
}
