import java.util.Random;
import java.util.ArrayList;

class Kmeans {

    public Kmeans() {
	this.random = new Random();	
    }

    /**
     *
     *  Create a point in the bounds
     */
    public Example randomPoint(DataFrame df) {
	Example point = new Example();

	for(MinMax bound : df.bounds) {
	    point.add(bound.min + random.nextDouble() * (bound.max - bound.min));
	}

	return point;
    }
    
    /**
     *
     *  Compute the k-means algorithm
     */
    public int[] run(DataFrame df, int k) {
	// for each points, the index of the cluster
	int [] cluster = new int[df.size()];

	// create the centers of the clusters
	Example [] centers = new Example[k];

	for(int i = 0; i < k; i++)
	    centers[i] = randomPoint(df);

	// stopping condition of the algorithm
	boolean stop = false;
	int c;

	while (!stop) {
	    stop = true;

	    // put every points in the nearest cluster
	    for(int i = 0; i < cluster.length; i++) {
		c = nearestCluster(df.getExample(i), centers);
		if (c != cluster[i]) {		
		    cluster[i] = c;
		    stop = false;
		}
	    }

	    // compute the new center of the clusters
	    updateCenters(df.getExamples(), cluster, centers);
	}

	return cluster;
    }

    /**
     *
     *  Compute the nearest center of the given point
     */
    private int nearestCluster(Example point, Example [] centers) {
	int nearest = 0;
	double dMin = distance(point, centers[0]);

	for(int j = 1; j < centers.length; j++) {
	    double d = distance(point, centers[j]);
	    if (d < dMin) {
		nearest = j;
		dMin = d;
	    }
	}

	return nearest;
    }

    /**
     *
     *  Compute the position of the centers
     *
     */
    private void updateCenters(ArrayList<Example> points, int [] clusters, Example [] centers) {
	// number of clusters
	int k = centers.length;
	// dimension of the space
	int dim = points.get(0).size();

	// to save old position in the case of no point in the cluster
	Example [] centersOld = new Example[k];

	// number of points in each cluster
	int [] nb = new int[k];

	for(int i = 0; i < k; i++) {
	    centersOld[i] = centers[i].clone();
	    for(int j = 0; j < dim; j++)	    
		centers[i].set(j, 0);
	    nb[i] = 0;
	}

	// tmp variables
	int c;
	Example point;
	
	// Cumulative sum for all points in the cluster
	for(int i = 0; i < points.size(); i++) {
	    c = clusters[i];

	    nb[c]++;

	    point = points.get(i);
	    for(int j = 0; j < dim; j++)
		centers[c].set(j, centers[c].get(j) + point.get(j));
	}

	// divided by the number of points in each cluster to compute the average.
	for(int i = 0; i < k; i++) {
	    if (nb[i] != 0)
		for(int j = 0; j < dim; j++)
		    centers[i].set(j, centers[i].get(j) / nb[i]);
	    else {
		// no points in the cluster, don't move it
		centers[i] = centersOld[i];
	    }
	}
    }

    /**
     *
     *  Compute the distance
     */
    private double distance(Example ex1, Example ex2) {
	double d = 0.0;

	for(int i = 0; i < ex1.size(); i++)
	    d += (ex1.get(i) - ex2.get(i)) * (ex1.get(i) - ex2.get(i));
	
	return Math.sqrt(d);
    }

    // random number generator
    private Random random;

}
